import java.util.concurrent.atomic.AtomicLong
import java.util.TimeZone

import org.hyperic.util.StringUtil
import org.hyperic.util.TimeUtil
import org.hyperic.hq.measurement.MeasurementConstants
import org.hyperic.util.pager.PageControl
import org.hyperic.hq.appdef.server.session.ConfigManagerEJBImpl
import org.hyperic.hq.appdef.shared.AppdefEntityID
import org.hyperic.hq.appdef.shared.ConfigFetchException
import org.hyperic.hq.authz.shared.PermissionManagerFactory
import org.hyperic.hq.authz.shared.PermissionException
import org.hyperic.hq.hqu.rendit.BaseController
import org.hyperic.hq.measurement.server.session.AvailabilityManagerEJBImpl
import org.hyperic.hq.measurement.server.session.DataManagerEJBImpl
import org.hyperic.hq.measurement.server.session.DataPoint
import org.hyperic.hq.product.ProductPlugin;
import org.hyperic.hq.authz.shared.AuthzConstants
import org.json.JSONArray
import org.json.JSONObject

class PerformanceController
    extends BaseController {
 
    private boolean hasPlatformPermission(instanceId) {
        try {
            PermissionManagerFactory.getInstance().check(user.getId(), 
                                                     AuthzConstants.platformResType, 
                                                     instanceId,
                                                     AuthzConstants.platformOpViewPlatform)
        } catch(PermissionException e) {
            return false
        }
        
        return true
    }

    private long adjustTimestamp(date, clientTimezoneOffset) {
        def timezoneOffset = TimeZone.getDefault().getOffset(date)
        
        clientTimezoneOffset *= -1
        
        return date - (clientTimezoneOffset - timezoneOffset)
    }
    
    private JSONObject getAvailabilityJSON(mid, range, utcOffset) {
        def availMan = AvailabilityManagerEJBImpl.one
        def m = getMetricHelper().findMeasurementById(mid)
        def end = now()
        def start = end - range
        def interval = TimeUtil.getInterval(start, end, 100)
        Integer[] mids = [ m.id ];
        def data = availMan.getHistoricalAvailData(mids, start, end, interval, PageControl.PAGE_ALL, true)
        def result = new JSONObject()
        
        result.put("start", adjustTimestamp(start, utcOffset))
        result.put("end", adjustTimestamp(end, utcOffset))
        
        def metrics = new JSONArray()
        
        data.each { dp ->
            def metric
            
            if (metrics.length() > 0) {
                metric = metrics.getJSONObject(metrics.length() - 1)
            }
            
            def code = dp.value
            def status = "yellow"
                
            if (code == MeasurementConstants.AVAIL_UNKNOWN) {
                status = "grey"
            } else if (code == MeasurementConstants.AVAIL_DOWN) {
                status = "red"
            } else if (code == MeasurementConstants.AVAIL_UP) {
                status = "green"
            } else if (code == MeasurementConstants.AVAIL_PAUSED) {
                status = "orange"
            } else if (code == MeasurementConstants.AVAIL_POWERED_OFF) {
                status = "black"
            }
    
            if (metric) {
                if (metric.get("status") == status) {
                    def counter = metric.getInt("units")
                    
                    metric.put("units", counter + 1)
                    metric.put("end", metric.getLong("start") + ((counter + 1) * interval))
                } else {
                    metric = null
                }
            }
            
            if (!metric) {
                def timestamp = dp.timestamp
                
                metric = new JSONObject()
                
                metric.put("status", status)
                metric.put("start", adjustTimestamp(timestamp - interval, utcOffset))
                metric.put("end", adjustTimestamp(timestamp, utcOffset))
                metric.put("units", 1)
                
                metrics.put(metric)
            }
        }
        
        result.put("status", metrics.getJSONObject(metrics.length() - 1).get("status"))
        result.put("metrics", metrics)  
        
        result
    }
    
    private String checkResourceConfiguration(resource) {
        def appdefEntityId = new AppdefEntityID(resource)
        def conMan = ConfigManagerEJBImpl.one
        
        try {
            conMan.getMergedConfigResponse(user, ProductPlugin.TYPE_MEASUREMENT, appdefEntityId, true)
        } catch(ConfigFetchException e) {
            return localeBundle.getFormattedMessage('text.resource.not.configured', 
                                                    invokeArgs.context.contextPath, 
                                                    String.valueOf(appdefEntityId.type), 
                                                    String.valueOf(appdefEntityId.id))
        }
        
        def configResponse = conMan.getConfigResponse(appdefEntityId)
        def validationError = configResponse.getValidationError()
            
        if (validationError) {
            validationError = StringUtil.replace(validationError, "\n", "<br>&nbsp;&nbsp;&nbsp;&nbsp;")
            
            return localeBundle.getFormattedMessage('text.resource.configuration.invalid', 
                                                    validationError, 
                                                    invokeArgs.context.contextPath, 
                                                    String.valueOf(appdefEntityId.type), 
                                                    String.valueOf(appdefEntityId.id))
        }
        
        return null
    }
    
    def index(params) {
        def id = params.getOne('id')?.toInteger()
        def range = params.getOne('range', "43200000").toLong()
        def utcOffset = params.getOne('utcoffset', "0").toLong()

        if (!id) {
            render(inline: "No resource selected")
            
            return
        }

        def host
        def hostMetrics 
        def vm
        def vmMetrics
        def resource = resourceHelper.findById(id)
        def resourceMetrics = []
        def associatedPlatform
        def availMetric
        def errorMessage = checkResourceConfiguration(resource)
        def availabilityJSON
        def canViewResource = false

        if (resource.resourceType.id == AuthzConstants.authzPlatform) {
            canViewResource = hasPlatformPermission(resource.getInstanceId())
        } else {
            // ...if the resource is a server the user can see it...
            canViewResource = true
        }

        if (!errorMessage) {
            def type = resource.prototype.name
            def ancestors = resourceHelper.findAncestorsByVirtualRelation(resource)       
             
            resource.getEnabledMetrics().each { metric ->
                if (!metric.template.isAvailability()) {
                    resourceMetrics << metric
                }
            }

            availMetric = resource.getAvailabilityMeasurement()
            
            if (availMetric) {
                availabilityJSON = new JSONObject()
                
                availabilityJSON.put("data", getAvailabilityJSON(availMetric.id, range, utcOffset))
            }
            
            if (type == AuthzConstants.platformPrototypeVmwareVsphereVm ||
                (type != AuthzConstants.serverPrototypeVmwareVcenter && resource.resourceType.id == AuthzConstants.authzServer)) {
                host = ancestors.find { res -> res.prototype.name == AuthzConstants.platformPrototypeVmwareVsphereHost }
                hostMetrics = [] // Metrics which this resource can be compared against
                           
                host.getEnabledMetrics().each { metric ->
                    if (!metric.template.isAvailability()) {
                        hostMetrics << [name: metric.template.name, id: metric.id]
                    }
                }
                
                def children = resourceHelper.findChildResourcesByVirtualRelation(resource)
                def platform = children.find({ res -> res.resourceType.id == AuthzConstants.authzPlatform })
                    
                if (platform && hasPlatformPermission(platform.getInstanceId())) {
                    associatedPlatform = platform
                }
            
                if (resource.resourceType.id == AuthzConstants.authzServer) {
                    vm = ancestors.find { res -> res.prototype.name == AuthzConstants.platformPrototypeVmwareVsphereVm }
                    vmMetrics = []
                                 
                    vm.getEnabledMetrics().each { metric ->
                        if (!metric.template.isAvailability()) {
                            vmMetrics << [name: metric.template.name, id: metric.id]
                        }
                    }
                }
            }
        }
        
        render(locals:[ resource : resource,
                        canViewResource : canViewResource,
                        host : host,
                        vm: vm,
                        availMetric: availMetric,
                        resourceMetrics: resourceMetrics,
                        associatedPlatform: associatedPlatform,
                        availabilityJSON: availabilityJSON,
                        vmMetrics: vmMetrics,
                        hostMetrics: hostMetrics,
                        errorMessage: errorMessage ])
    }

    def availability(params) {
        def mid = params.getOne('mid').toInteger()
        def range = params.getOne('range').toLong()
        def utcOffset = params.getOne('utcoffset', "0").toLong()
        def result = new JSONObject()
        
        result.put("data", getAvailabilityJSON(mid, range, utcOffset))
        
        render(inline:"${result}", contentType:'text/json-comment-filtered')
    }
    
    private JSONArray generateTimeSeries(start, end, interval, data, utcOffset) {
        def result = new JSONArray()
        def lastIndex = 0
        def adjustedStart = start
        def adjustedEnd = end
        
        if (!data.isEmpty()) {
            def actualEnd = data.get(0).timestamp
            def correction = adjustedEnd - actualEnd
            
            if (correction > 0) {
                if (correction > interval) {
                    correction = correction - (((long) Math.ceil(correction/interval)) * interval)
                }
                
                adjustedEnd = end - correction
                adjustedStart = start - correction
            }
        }

        def lastPointNull = false
        
        for (long x = adjustedEnd; x > adjustedStart - interval; x -= interval) {
            def point = new JSONArray()
            def timestamp
            def value
            
            point.put(adjustTimestamp(x, utcOffset))
            
            if (data.size() > lastIndex) {
                def dp = data.get(lastIndex)
                
                timestamp = dp.timestamp
                
                if (timestamp == x) {
                    value = dp.value
                    lastPointNull = false
                }
            }
            
            if (timestamp > x) {
                x += interval
                lastIndex++
                continue
            }
            
            point.put(value)
            
            if (value == null) {
                if (result.length() > 0 && lastPointNull) {
                    def lastDataPoint = result.getJSONArray(result.length() - 1)
                    
                    if (lastDataPoint.isNull(1)) {
                        // TODO Tiny hack to get around limitations of the json library
                        // Initially upgraded to latest json code to get the remove function
                        // but the wrap function implementation is quite buggy and we
                        // use it heavily in other parts of HQ.  Resorting to this 
                        // for the time being, to minimize impact.
                        result.myArrayList.remove(result.length() - 1)
                    }
                }
            }
            
            result.put(point)
            
            if (value == null) {
                lastPointNull = true
            }
        }

        // Adjust end and start points if null so that they have the appropriate timestamp
        if (result.length() > 0) {
            def endPoint = result.getJSONArray(0)
            def offsetEnd = adjustTimestamp(adjustedEnd, utcOffset)
            
            if (endPoint.isNull(1) && endPoint.get(0) < offsetEnd) {
                endPoint.put(0, offsetEnd)
            }
            
            def startPoint = result.getJSONArray(result.length() - 1)
            def offsetStart = adjustTimestamp(adjustedStart, utcOffset)
            
            if (startPoint.isNull(1) && startPoint.get(0) < offsetStart) {
                startPoint.put(0, offsetStart)
            }
        }
        
        result
    }
    
    def data(params) {
        def mid = params.getOne('mid').toInteger()
        def range = params.getOne('range').toLong()
        def utcOffset = params.getOne('utcoffset', '0').toLong()
        def compare = params.getOne('compare', '0').toInteger()
        def dataMan = DataManagerEJBImpl.one
        def m = getMetricHelper().findMeasurementById(mid)
        def end = now()
        def start = end - range
        AtomicLong publishedInterval = new AtomicLong()
        def data = dataMan.getRawData(m, start, end, publishedInterval)
        def res = new JSONObject();
        def resData = new JSONArray()
        def interval = (publishedInterval.get() == 0) ? m.interval : publishedInterval.get()
        def dataArray = generateTimeSeries(start, end, interval, data.sort({ a, b -> b.timestamp <=> a.timestamp }), utcOffset)
        def dataMap = [ 'label' : m.template.name, 'data' : dataArray, 'units': m.template.units ]
        
        resData.put(dataMap)
        res.put('start', adjustTimestamp(start, utcOffset))
        res.put('end', adjustTimestamp(end, utcOffset))
        res.put('y1units', m.template.units)

        if (compare != 0) { 
            def compareMetric
            
            if (compare == -1) {
                def ancestors = resourceHelper.findAncestorsByVirtualRelation(m.resource)
                def host = ancestors.find { r -> r.prototype.name == AuthzConstants.platformPrototypeVmwareVsphereHost }
                
                if (host) {
                    compareMetric = host.getEnabledMetrics().find({ metric -> metric.template.name == m.template.name })
                }
            } else {
                compareMetric = getMetricHelper().findMeasurementById(compare)
            }
            
            if (compareMetric) {
                AtomicLong comparePublishedInterval = new AtomicLong()
                def compareData = dataMan.getRawData(compareMetric, start, end, comparePublishedInterval)
                def compareInterval = (comparePublishedInterval.get() == 0) ? compareMetric.interval : comparePublishedInterval.get()
                def compareDataArray = generateTimeSeries(start, end, compareInterval, compareData.sort({ a, b -> b.timestamp <=> a.timestamp }), utcOffset)
                def compareDataMap = [ 'label' : compareMetric.template.name,
                                       'data': compareDataArray,
                                       'yaxis' : 2,
                                       'units' : compareMetric.template.units ]
                
                resData.put(compareDataMap)
                res.put('y2units', compareMetric.template.units)
            }
        }

        res.put('data', resData)

        render(inline:"${res}", contentType:'text/json-comment-filtered')
    }
}