// set up the dashboard widget namespace
hyperic = hyperic || {}; 
/** 
 * @namespace
 */
hyperic.dashboard = hyperic.dashboard || {}; 


/**
 * common hyperic dashboard widget functions
 *
 * @author Anton Stroganov <anton@hyperic.com>
 * @class hyperic.dashboard.widget
 * @requires hyperic.dashboard.widget
 */
hyperic.dashboard.widget = {
    configSheet: null,
    contentSheet: null,
    config: {},

    /**
     * catches clicks on widget, and passes them on to the handler function 
     * determined by the button's class.
     *
     * @param {Event} e
     * @see #clickHandler
     */
    clickHandler: function(e) {
        if(this['click_' + e.target.className])
        {
            e.stopPropagation();
            e.preventDefault();
            dojo11.stopEvent(e);

            this['click_' + e.target.className](e);
        }
    },
    
    /**
     * clicking the config button shows the config layer 
     * @param {Event} e
     * @see #clickHandler
     */
    click_config_btn: function(e,onEnd)
    {
        this.swapSheets(this.contentSheet,this.configSheet,onEnd);
    },

    /**
     * clicking the config layer's cancel button swaps config and 
     * content layers back
     *
     * @param {Event} e
     * @see #clickHandler
     */
    click_cancel_btn: function(e,onEnd)
    {
        this.swapSheets(this.configSheet,this.contentSheet,onEnd);
    },
    
    /**
     * clicking the config layer's save button should make an xhr request
     * to store the data, and swaps the content layer back in
     *
     * @param {Event} e
     * @see #clickHandler
     */
    click_save_btn: function(e)
    {
        throw new Error('You need to implement this widget\'s save config functionality!');
    },
    
    /**
     * cleanup code should happen in the child class, and then invoke this
     * to remove the dashboard portlet from dashboard page.
     */
    click_remove_btn: function(e)
    {
        removePortlet(this.config.portletName, this.config.portletLabel);
    },
    
    /**
     * swap any two layers with a dojo fade-out -> fade-in effect
     * invokes function passed in as 'onEnd' argument after the fade
     *
     * @param {Node} from
     * @param {Node} to
     * @param {function} onEnd
     */
    swapSheets: function(from,to,onEnd) {
        var c = dojo11.fx.chain([
            dojo11.fadeOut({
                node : from, 
                onEnd: function() { 
                        dojo11.style(from,'display','none'); 
                        dojo11.style(to,'display','block'); 
                    } 
                }
            ),
            dojo11.fadeIn({
                node : to
                }
            )
            ]);

        if(onEnd)
        {
            dojo11.connect(c,'onEnd',onEnd);
        }
        c.play();
    },

    /**
     * filter a selectbox to show only the options
     * with names that match the text given.
     *
     * @param {Node} selectbox node
     * @param {Text} search text
     */
    searchSelectBox: function(node,text) {
        dojo11.forEach(
            node.options,
            function(opt)
            {
                // if(opt.text.match(exp))
                if(opt.text.toLowerCase().indexOf(text.toLowerCase()) !== -1)
                {
                    if(opt.disabled === true)
                    {
                        opt.disabled = false;
                        opt.style.display = '';
                    }
                }
                else
                {
                    if(opt.disabled === false)
                    {
                        opt.disabled = true;
                        opt.style.display = 'none';
                    }
                }
            });
    },

    /**
     * add an option to a selectbox in a correct position.
     * to keep the box alphabetically sorted.
     *
     * @param {Node} selectbox node to add option to
     * @param {Node} new option to add do the selectbox
     */
    addOptionToSelect: function(select,option)
    {
        var newLocation = null;
        if(select.options.length > 0)
        {
            for(var i = 0,j = select.options.length; i < j; i++)
            {
                if(select.options[i].text > option.text)
                {
                    newLocation = i;
                    break;
                }
            }
        }
        try {  // standards compliant; doesn't work in IE
            if(null !== newLocation)
            {
                select.add(option, select.options[newLocation]);
            }
            else
            {
                select.add(option, null); // standards compliant; doesn't work in IE
            }
        }
        catch(ex) { // IE only
            if(null !== newLocation)
            {
                select.add(option,newLocation); // IE only
            }
            else
            {
                select.add(option); // IE only
            }
        }
    }
};

/**
 * chartWidget is a widget that displays a chart slideshow
 * 
 * @author Anton Stroganov <anton@hyperic.com>
 * @base hyperic.dashboard.widget
 * @constructor
 */
hyperic.dashboard.chartWidget = function(node, portletName, portletLabel) {
    var that = this;

    that.configSheet = dojo11.query('.config',node)[0];
    that.contentSheet = dojo11.query('.content',node)[0];
    
    that.chartsearch = dojo11.byId('chartsearch');
    that.chartselect = dojo11.byId('chartselect');
    
    that.play_btn = dojo11.query('.pause_btn',node)[0];
    
    that.chart = null;
    that.charts = [];
    that.cycleId = null;
    that.currentChartId = null;
    that.showing = 'content';
    that.needsResize = false;
    that.config = {
        portletName: portletName,
        portletLabel: portletLabel
    };

    /**
     * pause the chart playback before showing the config layer 
     * populate the config form from the config object
     * 
     * @param {Event} e
     * @see #playCharts
     * @see hyperic.dashboard.widget#click_config_btn
     * @see hyperic.dashboard.widget#clickHandler
     */
    this.click_config_btn = function(e)
    {
        that.pauseCharts();
        hyperic.dashboard.widget.click_config_btn.apply(this);
        that.showing = 'config';
        
        var input_rotation = dojo11.byId('chart_rotation');
        var input_interval = dojo11.byId('chart_interval');
        var input_range = dojo11.byId('chart_range');
        if(that.config.rotation == 'true')
        {
            input_rotation.checked = true;
        }
        for(var i = 0; i < input_interval.options.length; i++)
        {
            if(input_interval.options[i].value == that.config.interval)
            {
                input_interval.selectedIndex = i;
                break;
            }
        }
        for(var j = 0; j < input_range.options.length; j++)
        {
            if(input_range.options[j].value == that.config.range)
            {
                input_range.selectedIndex = j;
                break;
            }
        }
    };

    /**
     * sets the config values from the config form
     * TODO: makes request to store config on server
     * after saving and showing the content layer 
     * - resize chart if necessary
     * - restart chart slideshow playback if rotation is still on
     * 
     * @param {Event} e
     * @see #playCharts
     * @see hyperic.dashboard.widget#click_save_btn
     * @see hyperic.dashboard.widget#clickHandler
     */
    this.click_save_btn = function(e)
    {
        // if the time range has changed, reset the chart data and chart refresh cycle
        if(that.config.range && that.config.range != dojo11.byId('chart_range').value)
        {
            for(var i = 0; i < that.charts.length; i++)
            {
                if(that.charts[i].interval)
                {
                    clearInterval(that.charts[i].interval);
                    that.charts[i].interval = null;
                }
                that.charts[i].data = null;
            }
        }

        that.config.interval = parseInt(dojo11.byId('chart_interval').value,10);
        that.config.range = dojo11.byId('chart_range').value;
        that.config.rotation = dojo11.byId('chart_rotation').checked ? 'true' : 'false';

        dojo11.xhrGet( {
            url: "/api.shtml?v=1.0&s_id=chart&config=true&tr=" + that.config.range + "&ivl=" + that.config.interval + "&rot=" + that.config.rotation,
            handleAs: 'json',
            load: function(data){
                that.config.interval = parseInt(data.ivl,10) || that.config.interval;
                that.config.range = data.tr || that.config.range;
                that.config.rotation = data.rot || that.config.rotation;
            },
            error: function(data){
                console.debug("An error occurred saving charts config... ", data);
            },
            timeout: 2000
        });


        that.swapSheets(that.configSheet,that.contentSheet,
            function()
            {
                that.showing = 'content';
                if(that.needsResize)
                {
                    that.chartResize();
                }
                if(that.config.rotation == 'true')
                {
                    that.playCharts();
                }
            });
    };

    /**
     * extends parent's behaviour to restart chart slideshow playback
     * after canceling config and showing the content layer 
     * 
     * @param {Event} e
     * @see #playCharts
     * @see hyperic.dashboard.widget#click_cancel_btn
     * @see hyperic.dashboard.widget#clickHandler
     */
    this.click_cancel_btn = function(e)
    {
        hyperic.dashboard.widget.click_cancel_btn.apply(
            this,
            [
                e,
                function()
                {
                    that.showing = 'content';
                    if(that.needsResize)
                    {
                        that.chartResize();
                    }
                    if(that.config.rotation == 'true')
                    {
                        that.playCharts();
                    }
                }
            ]);
    };

    /**
     * a play button click handler to start playback, 
     * and change play button to a pause button. 
     * 
     * @param {Event} e
     * @see #playCharts
     * @see hyperic.dashboard.widget#clickHandler
     */
    that.click_play_btn = function(e) {
        that.playCharts();
    };
    
    /**
     * a pause button click handler to stop playback, 
     * and change pause button to a play button. 
     * 
     * @param {Event} e
     * @see #pauseCharts
     * @see hyperic.dashboard.widget#clickHandler
     */
    that.click_pause_btn = function(e) {
        that.pauseCharts();
    };
    
    /**
     * extends parent's behaviour to pause the chart playback 
     * and cleanup the chart before removing the widget
     * 
     * @param {Event} e
     * @see #pauseCharts
     * @see hyperic.dashboard.widget#click_remove_btn
     * @see hyperic.dashboard.widget#clickHandler
     */
    this.click_remove_btn = function(e)
    {
        that.pauseCharts();
        that.chart.cleanup();
        hyperic.dashboard.widget.click_remove_btn.apply(that);
    };

    /**
     * an event handler to handle onKeyUp events on a search textbox.
     * It filters the chart selectbox of the widget to show only the charts
     * with names that match the text being typed in.
     * 
     * @param {Event} e
     */
    that.search = function(e)
    {
        that.searchSelectBox(that.chartselect,e.target.value);
    };
    
    /**
     * an event handler to handle the onfocus event on the search textbox.
     * It empties it to prepare it for user's input.
     * 
     * @param {Event} e
     */
    that.emptySearch = function(e)
    {
        if(e.target.value == '[ Live Search ]')
        {
            e.target.value = '';
        }
    };
    
    /**
     * an event handler to handle the onblur event on the search textbox.
     * It resets the search textbox to instruction value if it's empty.
     * 
     * @param {Event} e
     */
    that.resetSearch = function(e) 
    {
        if(e.target.value == '')
        {
            e.target.value = '[ Live Search ]';
        }
    };
    
    /**
     * an event handler to handle the onclick event on the chart selectbox.
     * pauses chart playback and swaps the current chart for the newly 
     * selected chart.
     * 
     * @param {Event} e
     */
    that.select = function(e)
    {
        that.pauseCharts();
    
        if(that.currentChartId != e.target.value)
        {
            that.cycleCharts(e.target.value);
        }
    };

    /**
     * swaps a chart for the next chart in the list. 
     * NB: use 'that.' rather than 'this.' keyword here, because when
     * the browser window invokes this function, 'this.' is set to 
     * the window context, rather than the object context.
     *
     * @see #playCharts
     * @see #pauseCharts
     */
    that.cycleCharts = function(chartId)
    {
        // console.info('argument: ' + chartId);
        // console.info('next defined by default, 0');
        var next = 0;
        if(!chartId || chartId < 0 || chartId >= that.chartselect.options.length)
        {
            if(that.chartselect.selectedIndex !== -1 && that.chartselect.selectedIndex != that.chartselect.options.length-1)
            {
                // console.info('next defined by next element from select box selected index ' + that.chartselect.selectedIndex + ' + 1');
                next = that.chartselect.selectedIndex+1;
            }
        }
        else
        {
            // console.info('next defined by passing in argument ' + chartId);
            next = chartId;
        }
    
        if(that.chart !== null)
        {
            that.chart.cleanup();
        }
    
        // console.log('next is' + next);
    
        if(that.charts[next].data)
        {
            that.chart = new hyperic.widget.Chart('chart_container', that.charts[next]);
            that.chartselect.selectedIndex = next;
            that.currentChartId = next;
            chartId = null;
        }
        else
        {
            that.fetchChartData(next).addCallback(
                function()
                {
                    // add a callback to refresh the chart data in a minute
                    // we use a closure because we can't use the extended setInterval syntax to pass parameters
                    // to the function being called (doesn't work in IE)
                    that.charts[next].interval = setInterval(function(){that.fetchChartData(next)},36000);
    
                    if(that.charts[next].data)
                    {
                        that.chart = new hyperic.widget.Chart('chart_container', that.charts[next]);
                    }
                    else
                    {
                        that.chart = new hyperic.widget.Chart('chart_container', {data: {'0': [0]}, name: that.charts[next].name});
                    }
                    that.chartselect.selectedIndex = next;
                    that.currentChartId = next;
                    chartId = null;
                });
        }
    };
    
    /**
     * display the first chart if no chart is showing, 
     * and start the chart cycle a chart for the next chart in the list. 
     *
     * @see #pauseCharts
     * @see #cycleCharts
     */
    that.playCharts = function() {
        console.log('starting to play');
        that.cycleCharts();
        if(that.cycleId == null) {
            that.cycleId = setInterval(that.cycleCharts, parseInt(that.config.interval,10)*1000);

            // display pause button when playing
            that.play_btn.src = '/images/4.0/icons/control_pause.png';
            that.play_btn.className = 'pause_btn';
            that.play_btn.alt = 'pause slideshow';
        }
    };
    
    /**
     * if charts are playing, clear the interval to pause the playback 
     *
     * @see #playCharts
     * @see #cycleCharts
     */
    that.pauseCharts = function() {
        if(that.cycleId != null) {
            clearInterval(that.cycleId);
            that.cycleId = null;
            
            // display play button when pausing
            that.play_btn.src = '/images/4.0/icons/control_play.png';
            that.play_btn.className = 'play_btn';
            that.play_btn.alt = 'play slideshow';
        }
    };

    /**
     * fetch the chart data from server for a given chart
     */
    that.fetchAndPlayCharts = function()
    {
        console.log('fetching charts');
        
        dojo11.xhrGet( {
            url: "/api.shtml?v=1.0&s_id=chart",
            handleAs: 'json',
            load: function(data){
                that.charts = data;
                that.populateChartSelect();
                that.playCharts();
            },
            error: function(data){
                console.debug("An error occurred fetching charts: ", data);
            },
            timeout: 2000
        });
    };

    that.fetchChartData = function(chart)
    {
        console.log('fetching from url ' + "/api.shtml?v=1.0&s_id=chart&rid=" + that.charts[chart].rid + "&mtid=[" + that.charts[chart].mtid + "]");
        return dojo11.xhrGet( {
            url: "/api.shtml?v=1.0&s_id=chart&rid=" + that.charts[chart].rid + "&mtid=[" + that.charts[chart].mtid + "]",
            handleAs: 'json',
            load: function(data){
                // that.charts[chart].data = data;
                if(!data.error && data.length > 0)
                {
                    console.log('data fetched ('+data.length+' elements found)');
                    that.charts[chart].data = data[0].data;
                }
                else
                {
                    console.log('dataset empty');
                }
            },
            error: function(data){
                console.debug("An error occurred fetching charts config ", data);
            },
            timeout: 2000
        });
    };

    /**
     * fetch the stored config for the chart dashboard widget
     */
    that.fetchConfig = function()
    {
        // preset defaults
        that.config.interval = 60;
        that.config.range = '1h';
        that.config.rotation = 'true';

        dojo11.xhrGet( {
            url: "/api.shtml?v=1.0&s_id=chart&config=true",
            handleAs: 'json',
            load: function(data){
                that.config.interval = parseInt(data.ivl,10) || that.config.interval;
                that.config.range = data.tr || that.config.range;
                that.config.rotation = data.rot || that.config.rotation;
            },
            error: function(data){
                console.debug("An error occurred fetching charts config ", data);
            },
            timeout: 2000
        });
    };

    /**
     * populate the available and selected alert selectboxes
     * 
     * @see hyperic.dashboard.widget#addOptionToSelect
     */
    that.populateChartSelect = function()
    {
        for(var i = 0; i < that.charts.length; i++)
        {
            that.addOptionToSelect(that.chartselect, new Option(that.charts[i].name,i));
        }
    };
    
    /**
     * destroy the old chart, resize the chart container, and re-create the chart
     * invoked on window resize to ensure that chart always fits the widget.
     */
    that.chartResize = function()
    {
        if(that.showing == 'content')
        {
            that.chart.cleanup();
            dojo11.query('#chart_container',that.contentSheet)[0].style.width = that.contentSheet.offsetWidth - 130;
            that.chart = new hyperic.widget.Chart('chart_container', that.charts[that.currentChartId]);
            that.needsResize = false;
        }
        else
        {
            that.needsResize = true;
        }
    };

    if(that.chartsearch && that.chartselect)
    {
        // connect the onclick event of the whole widget to the clickHandler
        // function of this object, inherited from hyperic.dashboard.widget.
        dojo11.connect(node,'onclick',dojo11.hitch(that,'clickHandler'));

        // set up the event handlers for the live search box
        dojo11.connect(that.chartsearch,'onfocus', that.emptySearch);
        dojo11.connect(that.chartsearch,'onblur', that.resetSearch);
        dojo11.connect(that.chartsearch,'onkeyup',that.search);

        // set up the event handler for the select box
        dojo11.connect(that.chartselect,'onclick',that.select);
        
        // handle resizing of the window
        dojo11.connect(window,'onresize',dojo11.hitch(that, that.chartResize));

        dojo11.query('#chart_container',that.contentSheet)[0].style.width = that.contentSheet.offsetWidth - 130;
        // dojo11.query('#widget_chart',that.contentSheet)[0].style.width = that.contentSheet.offsetWidth - 130;

        that.fetchConfig();
        that.fetchAndPlayCharts();

        // while(true)
        // {
        //     if(undefined !typeof this.charts[0].data)
        //     {
        //         this.playCharts();
        //         break;
        //     }
        // }

        if(that.config.rotation == 'false')
        {
            this.pauseCharts();
        }
    }
};

// set the hyperic.dashboard.widget as the ancestor of the chartWidget class.
hyperic.dashboard.chartWidget.prototype = hyperic.dashboard.widget;

/**
 * summaryWidget is a widget that displays alert summaries
 * $
 * @author Anton Stroganov <anton@hyperic.com>
 * @base hyperic.dashboard.widget
 * @constructor
 */
hyperic.dashboard.summaryWidget = function(node, portletName, portletLabel) {
    var that = this;

    that.configSheet = dojo11.query('.config',node)[0];
    that.contentSheet = dojo11.query('.content',node)[0];
    
    that.alert_groups = {"data": {}, "count": 0};
    that.selected_alert_groups = [];
    that.alert_group_status = {};

    that.available_alert_groups = dojo11.byId('available_alert_groups');
    that.enabled_alert_groups = dojo11.byId('enabled_alert_groups');
    that.groupsearch = dojo11.byId('groupsearch');

    that.tables = {
        lcol: dojo11.query('.lcol table tbody',node)[0],
        rcol: dojo11.query('.rcol table tbody',node)[0]
    };

    that.config = {
        portletName: portletName,
        portletLabel: portletLabel
    };

    /**
     * an event handler to handle onKeyUp events on the search textbox.
     * filters the available and enabled alert selectboxes of the widget
     * to show only the alert groups with names that match the given text 
     * 
     * @param {Event} e
     */
    that.search = function(e)
    {
        that.searchSelectBox(that.available_alert_groups, e.target.value);
        that.searchSelectBox(that.enabled_alert_groups, e.target.value);
    };
    
    /**
     * an event handler to handle the onfocus event on the search textbox.
     * It empties it to prepare it for user's input.
     * 
     * @param {Event} e
     */
    that.emptySearch = function(e)
    {
        if(e.target.value == '[ Group Search ]')
        {
            e.target.value = '';
        }
    };
    
    /**
     * an event handler to handle the onblur event on the search textbox.
     * It resets the search textbox to instruction value if it's empty.
     * 
     * @param {Event} e
     */
    that.resetSearch = function(e) 
    {
        if(e.target.value == '')
        {
            e.target.value = '[ Group Search ]';
        }
    };

    /**
     * an event handler to handle the onclick event on the enable alert button.
     * moves the selected alert from the available alerts selectbox to the 
     * enabled alerts selectbox.
     * 
     * @param {Event} e
     * @see #moveAlert
     * @see #disableAlert
     */
    that.click_enable_alert_btn = function(e)
    {
        // if(that.available_alert_groups.selectedIndex != -1)
        // {
        //     that.selected_alert_groups.push(that.available_alert_groups.options[that.available_alert_groups.selectedIndex].value);
        // }
        that.moveAlert(that.available_alert_groups,that.enabled_alert_groups);
    };
    
    /**
     * an event handler to handle the onclick event on the disable alert button.
     * moves the selected alert from the enabled alerts selectbox to the 
     * available alerts selet.
     * 
     * @param {Event} e
     * @see #moveAlert
     * @see #enableAlert
     */
    that.click_disable_alert_btn = function(e)
    {
        // remove the selected index from the this.enabled_alert_groups array
        // if(that.enabled_alert_groups.selectedIndex != -1)
        // {
        //     that.selected_alert_groups.splice(
        //         that.selected_alert_groups.indexOf(
        //             that.enabled_alert_groups.options[
        //                 that.enabled_alert_groups.selectedIndex
        //                 ].value
        //         ),
        //         1
        //     );
        // }
        that.moveAlert(that.enabled_alert_groups,that.available_alert_groups);
    };

    /**
     * save button handler
     * store the data on server, then rebuild the selected_alert_groups local array
     * fetch the updated alert group status,
     * repaint the tables and
     * swap the config layer and show content layer again
     */
    that.click_save_btn = function(e)
    {
        that.selected_alert_groups = [];
        for(var i = 0, j = that.enabled_alert_groups.options.length; i < j; i++)
        {
            if(that.enabled_alert_groups.options[i])
            {
                that.selected_alert_groups.push(that.enabled_alert_groups.options[i].value);
            }
        }

        dojo11.xhrGet( {
            url: "/api.shtml?v=1.0&s_id=alert_summary&config=true&rid=[" + that.selected_alert_groups + "]",
            handleAs: 'json',
            load: function(data){
                that.selected_alert_groups = data.rid || that.selected_alert_groups;
                that.alert_groups.data = data.avail || that.alert_groups.data;
                that.alert_groups.count = data.count || that.alert_groups.count;

                that.fetchAlertGroupStatus().addCallback(function(){
                    that.repaintAlertGroups();
                    that.swapSheets(that.configSheet,that.contentSheet);
                });
            },
            error: function(data){
                console.debug("An error occurred saving alerts config... " + data);
                that.swapSheets(that.configSheet,that.contentSheet);
            },
            timeout: 2000
        });
    };

    /**
     * an event handler to handle the onclick event on the disable alert button.
     * moves the selected alert from the enabled alerts selectbox to the 
     * available alerts selet.
     * 
     * @param {Node} from: selectbox node to move option from 
     * @param {Node} to: selectbox node to move option to 
     * @see #enableAlert
     * @see #disableAlert
     * @see hyperic.dashboard.widget#addOptionToSelect
     */
    that.moveAlert = function(from,to)
    {
        if(from.selectedIndex != -1)
        {
            that.addOptionToSelect(to, from.options[from.selectedIndex]);
            from.remove(from.selectedIndex);
        }
    };

    /**
     * populate the available and selected alert selectboxes
     * 
     * @see hyperic.dashboard.widget#addOptionToSelect
     */
    that.populateAlertGroups = function()
    {
        for(var i in that.alert_groups.data)
        {
            var to = null;
            var alertOption = new Option(that.alert_groups.data[i],i);
            for(var j = 0; j < that.selected_alert_groups.length; j++)
            {
                if(that.selected_alert_groups[j] == i)
                {
                    to = that.enabled_alert_groups;
                    break;
                }
            }
            to = to || that.available_alert_groups;
            
            that.addOptionToSelect(to,alertOption);
        }
    };

    /**
     * utility function to sort alert data by alert group name
     */
    that.by_name = function(a,b)
    {
        return that.alert_groups.data[a] > that.alert_groups.data[b];
    };

    /**
     * destroy current alert tables, and call #paintAlertGroups() to re-paint them
     */
    that.repaintAlertGroups = function() 
    {
        for(var i in that.tables)
        {
            while(that.tables[i].lastChild) {
              that.tables[i].removeChild(that.tables[i].lastChild);
            }
        }
        that.paintAlertGroups();
    };

    /**
     * populate the html tables with the alerts based on the data in the #alert_group_status array
     */
    that.paintAlertGroups = function()
    {
        var groups = that.selected_alert_groups.sort(that.by_name);
        var half = Math.ceil(groups.length/2);
        
        var status = {
            'red'    : 'Failure',
            'green'  : 'OK',
            'yellow' : 'Warning',
            'gray'   : 'No Data'
        };

        for(var i = 0; i < groups.length; i++)
        {
            var table = (i < half) ? that.tables.lcol : that.tables.rcol;
            var row = table.insertRow(-1);
            
            row.className = ((i < half ? i : i - half) % 2 == 0) ? 'even' : 'odd';

            row.id = 'alertGroup:' + groups[i];
            var data = that.alert_group_status[groups[i]] || ['gray','gray'];
            var name = that.alert_groups.data[groups[i]];
            row.innerHTML = '<th scope="row">'+ name +'</th><td><img src="/images/4.0/icons/'+data[0]+'.png" alt="'+ status[data[0]] +'"></td><td><img src="/images/4.0/icons/'+data[1]+'.png"alt="'+ status[data[1]]+'"></td>';
            data = name = null;
        }
    };

    /**
     * fetch all available alert groups from server
     * the server should return a json object of the following form:
     * { 
     *     data: { 
     *         "id" : "name",
     *         ...
     *     },
     *     count: 10
     * }
     * the count element shall be the total number of alert groups available
     * (may be greater than number returned)
     *
     * @param {Number} page number
     * @param {String} string to search alerts for
     */
    // that.fetchAlertGroups = function(page, searchString)
    // {
    //     dojo11.xhrGet( {
    //         url: "/api.shtml?v=1.0&s_id=alert_summary&config=true",
    //         handleAs: 'json',
    //         load: function(data){
    //             that.alert_groups.data = data.avail || that.alert_groups.data;
    //             that.alert_groups.count = data.count || that.alert_groups.count;
    //             that.fetchAlertGroupStatus();
    //         },
    //         error: function(data){
    //             console.debug("An error occurred fetching alert groups... ", data);
    //         },
    //         timeout: 2000
    //     });
    // 
    //     // // offset = offset || 0;
    //     // that.alert_groups = { 
    //     //         data: { 
    //     //             1 : "Apache VHosts",
    //     //             2 : "HTTP Serivces",
    //     //             3 : "Linux Boxes",
    //     //             4 : "REST API",
    //     //             5 : "SF Data Center",
    //     //             6 : "Storage 1",
    //     //             7 : "WS API",
    //     //             8 : "Applications",
    //     //             9 : "CentOS Boxes",
    //     //             10 : "SuSE Boxes"
    //     //         },
    //     //         count: 10
    //     //     };
    // };

    /**
     * fetch the alert group status from server for currently selected alert groups
     * the server should return a json object of the following form:
     * { 
     *    "id" : ['resouce alert status','group alert status'],
     *    ...
     * }
     * the status is a letter code ('r' for red,'g' for green,'y' for yellow, or 'd' for data unavailable);
     */
    that.fetchAlertGroupStatus = function()
    {
        return dojo11.xhrGet( {
            url: "/api.shtml?v=1.0&s_id=alert_summary",
            handleAs: 'json',
            load: function(data){
                that.alert_group_status = data;
            },
            error: function(data){
                console.debug("An error occurred fetching alert groups status... ", data);
            },
            timeout: 2000
        });
        
        // that.alert_group_status = {
        //     '1': ['r','g'],
        //     '2': ['g','y'],
        //     '3': ['g','g'],
        //     '4': ['g','r'],
        //     '5': ['g','y'],
        //     // '6': ['g','g'],
        //     '7': ['g','g'],
        //     '8': ['g','g'],
        //     '9': ['g','g'],
        //     '10': ['g','g']
        // };
    };

    /**
     * fetch the stored selected alert groups for the dashboard widget
     * the server should return a json array of the alert group id's
     * [ 
     *    "id",
     *    ...
     * ]
     */
    that.fetchConfig = function()
    {
        return dojo11.xhrGet( {
            url: "/api.shtml?v=1.0&s_id=alert_summary&config=true",
            handleAs: 'json',
            load: function(data){
                that.selected_alert_groups = data.rid || [];
                that.alert_groups.data = data.data || that.alert_groups.data;
                // that.alert_groups.count = data.count || that.alert_groups.count;
            },
            error: function(data){
                console.debug("An error occurred fetching alert group config... ", data);
            },
            timeout: 2000
        });
        // that.selected_alert_groups = ['1','2','3','4','5','6','7'];
    };

    if(that.available_alert_groups && that.enabled_alert_groups)
    {
        // connect the onclick event of the whole widget to the clickHandler
        // function of this object, inherited from hyperic.dashboard.widget.
        dojo11.connect(node,'onclick',dojo11.hitch(that,'clickHandler'));

        // set up the event handlers for the live search box
        dojo11.connect(that.groupsearch,'onfocus', that.emptySearch);
        dojo11.connect(that.groupsearch,'onblur', that.resetSearch);
        dojo11.connect(that.groupsearch,'onkeyup',that.search);
        
        that.fetchConfig().addCallback(
            function() {
                that.fetchAlertGroupStatus().addCallback(
                    function() {
                        that.populateAlertGroups();
                        that.paintAlertGroups();
                    });
            });
    }
};

// set the hyperic.dashboard.widget as the ancestor of the chartWidget class.
hyperic.dashboard.summaryWidget.prototype = hyperic.dashboard.widget;

// initialize the chartwidget
dojo11.addOnLoad( function() {
    cw = new hyperic.dashboard.chartWidget(dojo11.byId('chartWidget'));
    sw = new hyperic.dashboard.summaryWidget(dojo11.byId('summaryWidget'));
});